// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                   (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Ranlux.h:	--- Conrete implementation of RandomNumberGenerator class
//
//	DESCRIPTION:
//		Class Ranlux provides an implementation of the abstract class
//		RandomNumberGenerator in Random.h. It is based in the Fortran
//		subroutine RANLUX in ranlux.f:
//		Subtract-and-borrow random number generator proposed by
//		Marsaglia and Zaman, implemented by F. James with the name
//		RCARRY in 1991, and later improved by Martin Luescher
//		in 1993 to produce "Luxury Pseudorandom Numbers".
//		Fortran 77 coded by F. James, 1993
//
//		M. Luscher, Computer Physics Communications  79 (1994) 100
//		F. James, Computer Physics Communications 79 (1994) 111
//
//	CHANGE HISTORY:
//		UL, 1999-05-27: 1.0.0: - first version
//		UL, 1999-06-08: 1.1.0: - added min()/max() methods
//		UL, 2002-03-02: 1.2.0: - extented class to give client control
//								 over type of range
//		UL, 2002-03-26: 1.3.0: - added minMaxRange();
//
//	AUTHOR:
//		U. Lammers (UL), Thu May 27 16:15:23 MET DST 1999
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/Ranlux.h,v 1.6 2008/09/26 16:38:55 eojero Exp $
//

#ifndef RANLUX_H
#define RANLUX_H

#include "Random.h"

class Ranlux : public virtual RandomNumberGenerator {
public:
	Ranlux(SophisticationLevel level = ALMOST_PERFECT,
		   RangeType rangeType = CLOSED,
		   unsigned seed = 0, double min = 0., double max = 1.);
		//
		//	PURPOSE:
		//		Construct a Ranlux object
		//
		//	ARGUMENTS:
		//		level	: Describes the level of sophistication
		//				  of the generator in terms of quality of the
		//				  returned random numbers, the following mapping
		//				  applies:
		//					QUICK_AND_DIRTY:
		//						(p=24): equivalent to the original RCARRY
		//								of Marsaglia and Zaman, very long
		//								period, but fails many tests.
		//					GOOD:
		//						(p=48): considerable improvement in
		//								quality over level 0, now passes
		//								the gap test, but still fails
		//								spectral test.
		//					REALLY_GOOD:
		//						(p=97): passes all known tests, but
		//								theoretically still defective.
		//					ALMOST_PERFECT:
		//						(p=223): DEFAULT VALUE.  Any theoretically
		//								 possible correlations have very
		//								 small chance of being observed.
		//					PERFECT:
		//						(p=389): highest possible luxury, all 24
		//								 bits chaotic.
		//		seed	: see Random.h
		//		rangeType: see Random.h
		//		min/max	: lower/upper bound of returned numbers
		//

	void sophisticationLevel(SophisticationLevel level);
	void rangeType(RangeType rangeType);
	void seed(unsigned seed);
		//
		//	PURPOSE:
		//		See Random.h
		//

	void min(double min);
	void max(double max);
	void minMaxRange(double min, double max, RangeType rangeType);
		//
		//	PURPOSE:
		//		See Random.h
		//		

	void reset();
		//
		//	PURPOSE:
		//		See Random.h
		//

	double rand();
		//
		//	PURPOSE:
		//		See Random.h

	virtual ~Ranlux() { }
		//
		//	PURPOSE:
		//		See Random.h
		//

private:
	double	_range;

	void checkLimits() const;
};

#endif
