// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SimpleScanner.cc--- Implements the SimpleScanner class which represents
//                          a simple scanner.
//
//	DESCRIPTION:
//
//	USAGE EXAMPLES:
//
//	CHANGE HISTORY:
//
//	AUTHOR:
//		M.Thomas
//
//	REFERENCES:
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/SimpleScanner.cc,v 1.7 2014/04/15 14:27:48 aibarra Exp $
///

#include <sstream>

#include "Tracer.h"
#include "SimpleScanner.h"
#include "stringutil.h"


SimpleScanner::Expected::Expected( const std::string& message )
	: _message( message )
{
	PROC
	ENDPROC
}


std::string
SimpleScanner::Expected::message() const
{
	PROC
	RETURN _message;
}


SimpleScanner::SimpleScanner(
	const std::string&	s,
	const std::string&	separators )
	:
		_source( s ),
		_ifs(*( new std::istringstream( _source.c_str() ))),
		_separators( separators )
{
	PROC
	fetchNextToken();
	ENDPROC
}


SimpleScanner::~SimpleScanner()
{
	PROC
	delete &_ifs;
	ENDPROC
}


bool
SimpleScanner::eof() const
{
	PROC
	//RETURN _ifs.eof();
	//RETURN _lookahead.id() == SimpleToken::Eof;
	RETURN _ifs.eof() && _lookahead.id() == SimpleToken::Eof;
}


const SimpleToken&
SimpleScanner::next() const
{
	PROC
	RETURN _lookahead;
}


SimpleToken
SimpleScanner::accept()
{
	PROC
	const SimpleToken	nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


void
SimpleScanner::skip()
{
	PROC
	fetchNextToken();
	ENDPROC
}


SimpleToken
SimpleScanner::match( SimpleToken::Id id ) throw( Expected )
{
	PROC
	if( _lookahead.id() != id )
	{
		throw Expected( std::string( "A token with token ID " ) + toString( int(id) ) + " (" + SimpleToken( id ).idAsString() + ") was expected" );
	} // if
	SimpleToken nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


SimpleToken
SimpleScanner::match( char c  ) throw( Expected )
{
	PROC
	if( _lookahead.id() != SimpleToken::Character || _lookahead.asChar() != c )
	{
		throw Expected( std::string( "The character '" ) + c + "' was expected" );
	} // if
	SimpleToken nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


bool
SimpleScanner::nextIsCharacter( char thisCharacter ) const
{
	PROC
	if(
		_lookahead.id() == SimpleToken::Character
		&&
		_lookahead.asChar() == thisCharacter )
	{
		ALTRETURNV( true );
	} // if
	RETURN false;
}


bool 
SimpleScanner::nextIsString() const
{
	PROC
	if( _lookahead.id() == SimpleToken::String )
	{
		ALTRETURNV( true );
	} // if
	RETURN false;
}


void
SimpleScanner::fetchNextToken()
{
	PROC
	if( !_ifs )
	{
		_lookahead = SimpleToken::Eof;
		ALTRETURN;
	} // if
	while( _ifs )
	{
		char	nextChar;
		_ifs >> nextChar;
		if( !_ifs )
		{
			_lookahead = SimpleToken::Eof;
		} 
		else if( _separators.find( nextChar ) == std::string::npos )
		{
			std::string	str;
			//str = str + nextChar;
			//_ifs >> nextChar;
			while( _ifs && _separators.find( nextChar ) == std::string::npos )
			{
				str = str + nextChar;
				_ifs >> nextChar;
			} // while
			//gcc4.8.2 portability. unget method changes the flagbit of eof()
			if (_ifs)
				_ifs.unget();

			_lookahead = str;
			break;
		}
		else
		{
			_lookahead = nextChar;
			break;
		} // if
	} // while
	ENDPROC
}


std::string
SimpleScanner::remainder() const
{
	PROC

	std::string	r;
	char 	c;

	switch( _lookahead.id() )
	{
		case SimpleToken::String :
			r = r + _lookahead.asString();
			break;
		case SimpleToken::Character :
			r = r + _lookahead.asChar();
			break;
		default :
			break;
	} // switch

	_ifs >> c;
	while( _ifs )
	{
		r = r + c;
	 	_ifs >> c;
	} // while
	RETURN r;
}



