// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*-----------------------------------------------------------------------------
 *
 *						XMM Science Analysis System (SAS)
 *					   (c) 1997 - 2010 European Space Agency
 *
 *-----------------------------------------------------------------------------
 *
 *	stime.h	--- C I/F to STime class
 *
 *	DESCRIPTION:
 *		See STime.h
 *
 *	CHANGE HISTORY:
 *		UL, 2001-07-12: 1.0.0: - first version
 *		UL, 2001-07-13: 1.0.1: - replaced ConstSTimeHandle with STimeHandle
 *								 to resolve problem with dynamic type checking
 *								 in SWIG generated wrapper code
 *
 *	AUTHOR:
 *		U. Lammers,	Thu May 10 14:10:42 MET DST 2001
 *
 *	REFERENCES:
 *		[1]: Software Specification Document for the XMM Science Analysis
 *			 Subsystem, M. Dahlem et. al.,
 *
 *	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/stime.h,v 1.3 2008/09/26 16:38:55 eojero Exp $
 */

#ifndef STIME_C_H
#define STIME_C_H

#include "common.h"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *----------------------------( STime )------------------------------------
 */
typedef void *				STimeHandle;

/*
 *	initSTime(): initialize STime module
 */
void initSTime();


/*
 *	construct new STime object
 *	no exception handling; conversion/out-of-range errors will result
 *	in error message and program termination
 */
STimeHandle ST_new0				();

STimeHandle ST_new1				(const char *stimeStr);
int			ST_isValidTime1		(const char *stimeStr);
STimeHandle ST_new2				(unsigned year, unsigned month,
								 unsigned day, unsigned hour,
								 unsigned minute, unsigned secs,
								 double excessSecs);
int			ST_isValidTime2		(unsigned year, unsigned month,
								 unsigned day, unsigned hour,
								 unsigned minute, unsigned secs,
								 double excessSecs);

typedef enum { JD = 0, MJD, DYN } ST_Representation;
STimeHandle	ST_new3				(double timeVal, ST_Representation r);

int			ST_isValidTime3		(double timeVal, ST_Representation r);

STimeHandle	ST_new4				(unsigned long timeValInt, double timeValFrac,
								 ST_Representation r);

/*	copy constructor */
STimeHandle	ST_clone			(STimeHandle time);

/* destructor */
void		ST_release			(STimeHandle t);


/*
 *	inquiry methods
 */
unsigned short ST_year			(STimeHandle t);/*	year number [1958, 2029] */
unsigned short ST_month			(STimeHandle t);/*	month number [1, 12]	 */
unsigned short ST_day			(STimeHandle t);/*	day number [1, 31]		 */
unsigned short ST_hour			(STimeHandle t);/*  hours elapsed since
													midnight [0, 23]		 */
unsigned short ST_minute		(STimeHandle t);/*	minutes elapsed since hour
												    [0, 59]					 */
unsigned short ST_seconds		(STimeHandle t);/*	seconds since minute
													[0, 60] (allow one leap
													second)					 */
unsigned short ST_weekday		(STimeHandle t);/*	0=Sun, 1=Mon, etc.		 */
double		ST_fracSeconds		(STimeHandle t);/*	0 <= fracSeconds <= 1	 */
unsigned short ST_leapSeconds	(STimeHandle t);/*	number of leap seconds	 */

/*
 *	member operators
 */
STimeHandle	ST_assign			(STimeHandle lhs, STimeHandle rhs);
												/* assignment				 */
STimeHandle ST_addTo			(STimeHandle t,	double s);
STimeHandle ST_subtractFrom		(STimeHandle t, double s);
												/* add/subtract time offset  */
												/* ... in seconds			 */

/*
 *	other member functions
 */
STimeHandle		ST_currentTime	(int asGMT);	/* return current-time object
												 * as GMT (asGMT=true) or
												 * local time (asGMT=false)	 */

typedef enum { UTC = 0,	/* Coordinated Universal Time */
			   TAI,		/*  International Atomic Time	*/
			   TT		/*  Terrestrial Time			*/
			 } ST_TimeSystem;

typedef void *					JDNumberHandle;

JDNumberHandle	ST_newJulianDay		(unsigned long day, double fractionalDay);
unsigned long	ST_julianDayDay		(JDNumberHandle julday);
void			ST_julianDaySetDay	(JDNumberHandle julday,
									 unsigned long daynr);
double			ST_julianDayFracDay	(JDNumberHandle julday);
void			ST_julianDaySetFracDay(JDNumberHandle julday, double frac);
double			ST_julianDayFull	(JDNumberHandle julday);
void			ST_julianDayRelease	(JDNumberHandle julday);

/* return JD/MJD number */
JDNumberHandle	ST_julianDay		(STimeHandle t, ST_TimeSystem timeSystem);
JDNumberHandle	ST_modifiedJulianDay(STimeHandle t, ST_TimeSystem timeSystem);

double			ST_siderialTime		(STimeHandle t, double longitude);

typedef enum { FITS = 0, CALENDAR } ST_StringRepresentation;

char *			ST_toString			(STimeHandle t,
									 unsigned fracSecondDigits,
									 ST_StringRepresentation form);

extern double	ST_secondsPerMinute;
extern double	ST_secondsPerHour;
extern double	ST_secondsPerDay;

extern STimeHandle	ST_earliest;
extern STimeHandle	ST_latest;

STimeHandle		ST_add				(STimeHandle t, double s);
STimeHandle		ST_sub				(STimeHandle t, double s);
double			ST_diff				(STimeHandle t1, STimeHandle t2);

int				ST_isLaterThan		(STimeHandle t1, STimeHandle t2);
int				ST_isLaterThanOrSameInstant
									(STimeHandle t1, STimeHandle t2);
int				ST_isEarlierThan	(STimeHandle t1, STimeHandle t2);
int				ST_isEarlierThanOrSameInstant
									(STimeHandle t1,STimeHandle t2);
int				ST_isNotSameInstant	(STimeHandle t1, STimeHandle t2);
int				ST_isSameInstant	(STimeHandle t1, STimeHandle t2);

#if defined(__cplusplus)
}
#endif

#endif
